@@ -23,6 +23,7 @@
 #include "sql_priv.h"
 #include "unireg.h"
 #include "sql_load.h"
+#include "sql_load.h"
 #include "sql_cache.h"                          // query_cache_*
 #include "sql_base.h"          // fill_record_n_invoke_before_triggers
 #include <my_dir.h>
@@ -682,6 +683,7 @@ int mysql_load(THD *thd, const sql_exchange *ex, TABLE_LIST *table_list,
     table->copy_blobs=1;
 
     thd->abort_on_warning= !ignore && thd->is_strict_mode();
+    thd->get_stmt_da()->reset_current_row_for_warning(1);
 
     bool create_lookup_handler= handle_duplicates != DUP_ERROR;
     if ((table_list->table->file->ha_table_flags() & HA_DUPLICATE_POS))
@@ -1013,7 +1015,8 @@ read_fixed_length(THD *thd, COPY_INFO &info, TABLE_LIST *table_list,
     read_info.row_end[0]=0;
 #endif
 
-    restore_default_record_for_insert(table);
+    restore_record(table, s->default_values);
+
     while ((item= it++))
     {
       Load_data_outvar *dst= item->get_load_data_outvar();
@@ -1126,8 +1129,8 @@ read_sep_field(THD *thd, COPY_INFO &info, TABLE_LIST *table_list,
                             thd->progress.max_counter);
       }
     }
+    restore_record(table, s->default_values);
 
-    restore_default_record_for_insert(table);
     while ((item= it++))
     {
       uint length;
@@ -1281,7 +1284,8 @@ read_xml_field(THD *thd, COPY_INFO &info, TABLE_LIST *table_list,
     }
 #endif
     
-    restore_default_record_for_insert(table);
+    restore_record(table, s->default_values);
+    
     while ((item= it++))
     {
       /* If this line is to be skipped we don't want to fill field or var */
