@@ -46,6 +46,18 @@ Created 10/21/1995 Heikki Tuuri
 #include <time.h>
 #endif /* !_WIN32 */
 
+/** The maximum size of a read or write request.
+
+According to Linux "man 2 read" and "man 2 write" this applies to
+both 32-bit and 64-bit systems.
+
+On FreeBSD, the limit is close to the Linux one, INT_MAX.
+
+On Microsoft Windows, the limit is UINT_MAX (4 GiB - 1).
+
+On other systems, the limit typically is up to SSIZE_T_MAX. */
+static constexpr unsigned os_file_request_size_max= 0x7ffff000;
+
 extern bool	os_has_said_disk_full;
 
 /** File offset in bytes */
@@ -107,36 +119,23 @@ struct pfs_os_file_t
 #endif
 };
 
-/** The next value should be smaller or equal to the smallest sector size used
-on any disk. A log block is required to be a portion of disk which is written
-so that if the start and the end of a block get written to disk, then the
-whole block gets written. This should be true even in most cases of a crash:
-if this fails for a log block, then it is equivalent to a media failure in the
-log. */
-
-#define OS_FILE_LOG_BLOCK_SIZE		512U
-
 /** Options for os_file_create_func @{ */
 enum os_file_create_t {
-	OS_FILE_OPEN = 51,		/*!< to open an existing file (if
-					doesn't exist, error) */
-	OS_FILE_CREATE,			/*!< to create new file (if
-					exists, error) */
-	OS_FILE_OVERWRITE,		/*!< to create a new file, if exists
-					the overwrite old file */
-	OS_FILE_OPEN_RAW,		/*!< to open a raw device or disk
-					partition */
-	OS_FILE_CREATE_PATH,		/*!< to create the directories */
-	OS_FILE_OPEN_RETRY,		/*!< open with retry */
+  /** create a new file */
+  OS_FILE_CREATE= 0,
+  /** open an existing file */
+  OS_FILE_OPEN,
+  /** retry opening an existing file */
+  OS_FILE_OPEN_RETRY,
+  /** open a raw block device */
+  OS_FILE_OPEN_RAW,
 
-	/** Flags that can be combined with the above values. Please ensure
-	that the above values stay below 128. */
+  /** do not display diagnostic messages */
+  OS_FILE_ON_ERROR_SILENT= 4,
 
-	OS_FILE_ON_ERROR_NO_EXIT = 128,	/*!< do not exit on unknown errors */
-	OS_FILE_ON_ERROR_SILENT = 256	/*!< don't print diagnostic messages to
-					the log unless it is a fatal error,
-					this flag is only used if
-					ON_ERROR_NO_EXIT is set */
+  OS_FILE_CREATE_SILENT= OS_FILE_CREATE | OS_FILE_ON_ERROR_SILENT,
+  OS_FILE_OPEN_SILENT= OS_FILE_OPEN | OS_FILE_ON_ERROR_SILENT,
+  OS_FILE_OPEN_RETRY_SILENT= OS_FILE_OPEN_RETRY | OS_FILE_ON_ERROR_SILENT
 };
 
 static const ulint OS_FILE_READ_ONLY = 333;
@@ -150,7 +149,7 @@ static const ulint OS_FILE_READ_ALLOW_DELETE = 555;
 /** Types for file create @{ */
 static constexpr ulint OS_DATA_FILE = 100;
 static constexpr ulint OS_LOG_FILE = 101;
-#if defined _WIN32 || defined HAVE_FCNTL_DIRECT
+#if defined _WIN32 || defined O_DIRECT
 static constexpr ulint OS_DATA_FILE_NO_O_DIRECT = 103;
 #endif
 /* @} */
@@ -211,20 +210,19 @@ public:
                       buf_tmp_buffer_t *slot= nullptr) :
     bpage(bpage), slot(slot), type(type) {}
 
-  bool is_read() const noexcept { return (type & READ_SYNC) != 0; }
-  bool is_write() const noexcept { return (type & WRITE_SYNC) != 0; }
-  bool is_async() const noexcept
-  { return (type & (READ_SYNC ^ READ_ASYNC)) != 0; }
+  bool is_read() const { return (type & READ_SYNC) != 0; }
+  bool is_write() const { return (type & WRITE_SYNC) != 0; }
+  bool is_async() const { return (type & (READ_SYNC ^ READ_ASYNC)) != 0; }
 
-  void write_complete(int io_error) const noexcept;
-  void read_complete(int io_error) const noexcept;
-  void fake_read_complete(os_offset_t offset) const noexcept;
+  void write_complete(int io_error) const;
+  void read_complete(int io_error) const;
+  void fake_read_complete(os_offset_t offset) const;
 
   /** If requested, free storage space associated with a section of the file.
   @param off   byte offset from the start (SEEK_SET)
   @param len   size of the hole in bytes
   @return DB_SUCCESS or error code */
-  dberr_t maybe_punch_hole(os_offset_t off, ulint len) noexcept
+  dberr_t maybe_punch_hole(os_offset_t off, ulint len)
   {
     return off && len && node && (type & (PUNCH ^ WRITE_ASYNC))
       ? punch_hole(off, len)
@@ -236,7 +234,7 @@ private:
   @param off   byte offset from the start (SEEK_SET)
   @param len   size of the hole in bytes
   @return DB_SUCCESS or error code */
-  dberr_t punch_hole(os_offset_t off, ulint len) const noexcept;
+  dberr_t punch_hole(os_offset_t off, ulint len) const;
 
 public:
   /** Page to be written on write operation */
@@ -319,7 +317,8 @@ struct os_file_stat_t {
 the temporary file is created in the in the mysql server configuration
 parameter (--tmpdir).
 @return temporary file handle, or NULL on error */
-FILE *os_file_create_tmpfile() noexcept;
+FILE*
+os_file_create_tmpfile();
 
 /**
 This function attempts to create a directory named pathname. The new directory
@@ -331,8 +330,10 @@ fail_if_exists arguments is true.
 @param[in]	fail_if_exists	if true, pre-existing directory is treated
 				as an error.
 @return true if call succeeds, false on error */
-bool os_file_create_directory(const char *pathname, bool fail_if_exists)
-  noexcept;
+bool
+os_file_create_directory(
+	const char*	pathname,
+	bool		fail_if_exists);
 
 /** NOTE! Use the corresponding macro os_file_create_simple(), not directly
 this function!
@@ -348,16 +349,16 @@ A simple function to open or create a file.
 pfs_os_file_t
 os_file_create_simple_func(
 	const char*	name,
-	ulint		create_mode,
+	os_file_create_t create_mode,
 	ulint		access_type,
 	bool		read_only,
-	bool*		success) noexcept;
+	bool*		success);
 
 /** NOTE! Use the corresponding macro
 os_file_create_simple_no_error_handling(), not directly this function!
 A simple function to open or create a file.
 @param[in]	name		name of the file or path as a null-terminated string
-@param[in]	create_mode	create mode
+@param[in]	create_mode	OS_FILE_CREATE or OS_FILE_OPEN
 @param[in]	access_type	OS_FILE_READ_ONLY, OS_FILE_READ_WRITE, or
 				OS_FILE_READ_ALLOW_DELETE; the last option
 				is used by a backup program reading the file
@@ -368,34 +369,18 @@ A simple function to open or create a file.
 pfs_os_file_t
 os_file_create_simple_no_error_handling_func(
 	const char*	name,
-	ulint		create_mode,
+	os_file_create_t create_mode,
 	ulint		access_type,
 	bool		read_only,
-	bool*		success) noexcept
+	bool*		success)
 	MY_ATTRIBUTE((warn_unused_result));
 
-#ifndef HAVE_FCNTL_DIRECT
-#define os_file_set_nocache(fd, file_name, operation_name) do{}while(0)
-#else
-/** Tries to disable OS caching on an opened file descriptor.
-@param[in]	fd		file descriptor to alter
-@param[in]	file_name	file name, used in the diagnostic message
-@param[in]	name		"open" or "create"; used in the diagnostic
-				message */
-void
-os_file_set_nocache(
-/*================*/
-	int	fd,		/*!< in: file descriptor to alter */
-	const char*	file_name,
-	const char*	operation_name) noexcept;
-#endif
-
 #ifndef _WIN32 /* On Microsoft Windows, mandatory locking is used */
 /** Obtain an exclusive lock on a file.
 @param fd      file descriptor
 @param name    file name
 @return 0 on success */
-int os_file_lock(int fd, const char *name) noexcept;
+int os_file_lock(int fd, const char *name);
 #endif
 
 /** NOTE! Use the corresponding macro os_file_create(), not directly
@@ -412,10 +397,10 @@ Opens an existing file or creates a new.
 pfs_os_file_t
 os_file_create_func(
 	const char*	name,
-	ulint		create_mode,
+	os_file_create_t create_mode,
 	ulint		type,
 	bool		read_only,
-	bool*		success) noexcept
+	bool*		success)
 	MY_ATTRIBUTE((warn_unused_result));
 
 /** Deletes a file. The file has to be closed before calling this.
@@ -453,7 +438,6 @@ bool os_file_close_func(os_file_t file);
 
 /* Keys to register InnoDB I/O with performance schema */
 extern mysql_pfs_key_t	innodb_data_file_key;
-extern mysql_pfs_key_t	innodb_log_file_key;
 extern mysql_pfs_key_t	innodb_temp_file_key;
 
 /* Following four macros are instumentations to register
@@ -610,7 +594,7 @@ pfs_os_file_t
 pfs_os_file_create_simple_func(
 	mysql_pfs_key_t key,
 	const char*	name,
-	ulint		create_mode,
+	os_file_create_t create_mode,
 	ulint		access_type,
 	bool		read_only,
 	bool*		success,
@@ -626,7 +610,7 @@ monitor file creation/open.
 @param[in]	key		Performance Schema Key
 @param[in]	name		name of the file or path as a null-terminated
 				string
-@param[in]	create_mode	create mode
+@param[in]	create_mode	OS_FILE_CREATE or OS_FILE_OPEN
 @param[in]	access_type	OS_FILE_READ_ONLY, OS_FILE_READ_WRITE, or
 				OS_FILE_READ_ALLOW_DELETE; the last option is
 				used by a backup program reading the file
@@ -641,7 +625,7 @@ pfs_os_file_t
 pfs_os_file_create_simple_no_error_handling_func(
 	mysql_pfs_key_t key,
 	const char*	name,
-	ulint		create_mode,
+	os_file_create_t create_mode,
 	ulint		access_type,
 	bool		read_only,
 	bool*		success,
@@ -668,7 +652,7 @@ pfs_os_file_t
 pfs_os_file_create_func(
 	mysql_pfs_key_t key,
 	const char*	name,
-	ulint		create_mode,
+	os_file_create_t create_mode,
 	ulint		type,
 	bool		read_only,
 	bool*		success,
@@ -857,7 +841,9 @@ to original un-instrumented file I/O APIs */
 @param[in]	file		handle to a file
 @return file size if OK, else set m_total_size to ~0 and m_alloc_size
 	to errno */
-os_file_size_t os_file_get_size(const char *filename) noexcept
+os_file_size_t
+os_file_get_size(
+	const char*	filename)
 	MY_ATTRIBUTE((warn_unused_result));
 
 /** Determine the logical size of a file.
@@ -872,7 +858,9 @@ os_offset_t os_file_get_size(os_file_t file) noexcept
 /** Truncates a file at its current position.
 @param[in/out]	file	file to be truncated
 @return true if success */
-bool os_file_set_eof(FILE *file) noexcept;
+bool
+os_file_set_eof(
+	FILE*		file);	/*!< in: file to be truncated */
 
 /** Truncate a file to a specified size in bytes.
 @param[in]	pathname	file path
@@ -885,14 +873,16 @@ os_file_truncate(
 	const char*	pathname,
 	os_file_t	file,
 	os_offset_t	size,
-	bool		allow_shrink = false) noexcept;
+	bool		allow_shrink = false);
 
 /** NOTE! Use the corresponding macro os_file_flush(), not directly this
 function!
 Flushes the write buffers of a given file to the disk.
 @param[in]	file		handle to a file
 @return true if success */
-bool os_file_flush_func(os_file_t file) noexcept;
+bool
+os_file_flush_func(
+	os_file_t	file);
 
 /** Retrieves the last error number if an error occurs in a file io function.
 The number should be retrieved before any other OS calls (because they may
@@ -904,7 +894,7 @@ the OS error number + OS_FILE_ERROR_MAX is returned.
                                         to the log
 @return error number, or OS error number + OS_FILE_ERROR_MAX */
 ulint os_file_get_last_error(bool report_all_errors,
-                             bool on_error_silent= false) noexcept;
+                             bool on_error_silent= false);
 
 /** NOTE! Use the corresponding macro os_file_read(), not directly this
 function!
@@ -923,7 +913,7 @@ os_file_read_func(
 	void*			buf,
 	os_offset_t		offset,
 	ulint			n,
-	ulint*			o) noexcept
+	ulint*			o)
 	MY_ATTRIBUTE((warn_unused_result));
 
 /** Rewind file to its start, read at most size - 1 bytes from it to str, and
@@ -936,7 +926,7 @@ void
 os_file_read_string(
 	FILE*		file,
 	char*		str,
-	ulint		size) noexcept;
+	ulint		size);
 
 /** NOTE! Use the corresponding macro os_file_write(), not directly this
 function!
@@ -966,7 +956,7 @@ bool
 os_file_status(
 	const char*	path,
 	bool*		exists,
-	os_file_type_t* type) noexcept;
+	os_file_type_t* type);
 
 /** This function reduces a null-terminated full remote path name into
 the path that is sent by MySQL for DATA DIRECTORY clause.  It replaces
@@ -980,30 +970,34 @@ This function manipulates that path in place.
 If the path format is not as expected, just return.  The result is used
 to inform a SHOW CREATE TABLE command.
 @param[in,out]	data_dir_path		Full path/data_dir_path */
-void os_file_make_data_dir_path(char *data_dir_path) noexcept;
+void
+os_file_make_data_dir_path(
+	char*	data_dir_path);
 
 /** Create all missing subdirectories along the given path.
 @return DB_SUCCESS if OK, otherwise error code. */
-dberr_t os_file_create_subdirs_if_needed(const char* path) noexcept;
+dberr_t
+os_file_create_subdirs_if_needed(
+	const char*	path);
 
 #ifdef UNIV_ENABLE_UNIT_TEST_GET_PARENT_DIR
 /* Test the function os_file_get_parent_dir. */
 void
-unit_test_os_file_get_parent_dir() noexcept;
+unit_test_os_file_get_parent_dir();
 #endif /* UNIV_ENABLE_UNIT_TEST_GET_PARENT_DIR */
 
 /**
 Initializes the asynchronous io system. */
-int os_aio_init() noexcept;
+int os_aio_init();
 
 /**
 Frees the asynchronous io system. */
-void os_aio_free() noexcept;
+void os_aio_free();
 
 /** Submit a fake read request during crash recovery.
 @param type   fake read request
 @param offset additional context */
-void os_fake_read(const IORequest &type, os_offset_t offset) noexcept;
+void os_fake_read(const IORequest &type, os_offset_t offset);
 
 /** Request a read or write.
 @param type		I/O request
@@ -1012,34 +1006,36 @@ void os_fake_read(const IORequest &type, os_offset_t offset) noexcept;
 @param n		number of bytes
 @retval DB_SUCCESS if request was queued successfully
 @retval DB_IO_ERROR on I/O error */
-dberr_t os_aio(const IORequest &type, void *buf, os_offset_t offset, size_t n)
-  noexcept;
+dberr_t os_aio(const IORequest &type, void *buf, os_offset_t offset, size_t n);
 
 /** @return number of pending reads */
-size_t os_aio_pending_reads() noexcept;
+size_t os_aio_pending_reads();
 /** @return approximate number of pending reads */
-size_t os_aio_pending_reads_approx() noexcept;
+size_t os_aio_pending_reads_approx();
 /** @return number of pending writes */
-size_t os_aio_pending_writes() noexcept;
+size_t os_aio_pending_writes();
 
 /** Wait until there are no pending asynchronous writes.
 @param declare  whether the wait will be declared in tpool */
-void os_aio_wait_until_no_pending_writes(bool declare) noexcept;
+void os_aio_wait_until_no_pending_writes(bool declare);
 
 /** Wait until all pending asynchronous reads have completed.
 @param declare  whether the wait will be declared in tpool */
-void os_aio_wait_until_no_pending_reads(bool declare) noexcept;
+void os_aio_wait_until_no_pending_reads(bool declare);
 
 /** Prints info of the aio arrays.
 @param[in/out]	file		file where to print */
-void os_aio_print(FILE *file) noexcept;
+void
+os_aio_print(FILE* file);
 
 /** Refreshes the statistics used to print per-second averages. */
-void os_aio_refresh_stats() noexcept;
+void
+os_aio_refresh_stats();
 
 /** Checks that all slots in the system have been freed, that is, there are
 no pending io operations. */
-bool os_aio_all_slots_free() noexcept;
+bool
+os_aio_all_slots_free();
 
 
 /** This function returns information about the specified file
@@ -1054,7 +1050,12 @@ os_file_get_status(
 	const char*	path,
 	os_file_stat_t* stat_info,
 	bool		check_rw_perm,
-	bool		read_only) noexcept;
+	bool		read_only);
+
+/** Set the file create umask
+@param[in]	umask		The umask to use for file creation. */
+void
+os_file_set_umask(ulint umask);
 
 #ifdef _WIN32
 
@@ -1065,7 +1066,7 @@ Make file sparse, on Windows.
 @param[in]	is_sparse if true, make file sparse,
 			otherwise "unsparse" the file
 @return true on success, false on error */
-bool os_file_set_sparse_win32(os_file_t file, bool is_sparse = true) noexcept;
+bool os_file_set_sparse_win32(os_file_t file, bool is_sparse = true);
 
 /**
 Changes file size on Windows
@@ -1082,12 +1083,14 @@ If file is normal, file system allocates storage.
 @param[in]	file		file handle
 @param[in]	size		size to preserve in bytes
 @return true if success */
-bool os_file_set_size(const char *pathname, os_file_t file, os_offset_t size)
-  noexcept;
+bool
+os_file_set_size(
+	const char*	pathname,
+	os_file_t	file,
+	os_offset_t	size);
 
 inline bool
 os_file_set_size(const char* name, os_file_t file, os_offset_t size, bool)
-  noexcept
 {
   return os_file_set_size(name, file, size);
 }
@@ -1111,14 +1114,14 @@ dberr_t
 os_file_punch_hole(
 	os_file_t	fh,
 	os_offset_t	off,
-	os_offset_t	len) noexcept
+	os_offset_t	len)
 	MY_ATTRIBUTE((warn_unused_result));
 
 /* Determine if a path is an absolute path or not.
 @param[in]	OS directory or file path to evaluate
 @retval true if an absolute path
 @retval false if a relative path */
-inline bool is_absolute_path(const char *path) noexcept
+inline bool is_absolute_path(const char *path)
 {
   switch (path[0]) {
 #ifdef _WIN32
