@@ -182,13 +182,9 @@ note that the trx may have been committed before the caller acquires
 trx_t::mutex
 @retval	NULL if no match */
 trx_t* trx_get_trx_by_xid(const XID* xid);
-/**********************************************************************//**
-If required, flushes the log to disk if we called trx_commit_for_mysql()
-with trx->flush_log_later == TRUE. */
-void
-trx_commit_complete_for_mysql(
-/*==========================*/
-	trx_t*	trx);	/*!< in/out: transaction */
+/** Durably write log until trx->commit_lsn
+(if trx_t::commit_in_memory() was invoked with flush_log_later=true). */
+void trx_commit_complete_for_mysql(trx_t *trx);
 /**********************************************************************//**
 Marks the latest SQL statement ended. */
 void
@@ -341,32 +337,28 @@ struct trx_lock_t
 
 #if  defined(UNIV_DEBUG) || !defined(DBUG_OFF)
   /** 2=high priority WSREP thread has marked this trx to abort;
-  1=another transaction chose this as a victim in deadlock resolution. */
+  1=another transaction chose this as a victim in deadlock resolution.
+
+  Other threads than the one that is executing the transaction may set
+  flags in this while holding lock_sys.wait_mutex. */
   Atomic_relaxed<byte> was_chosen_as_deadlock_victim;
 
   /** Flag the lock owner as a victim in Galera conflict resolution. */
   void set_wsrep_victim()
   {
-# if defined __GNUC__ && (defined __i386__ || defined __x86_64__)
-    /* There is no 8-bit version of the 80386 BTS instruction.
-    Technically, this is the wrong addressing mode (16-bit), but
-    there are other data members stored after the byte. */
-    __asm__ __volatile__("lock btsw $1, %0"
-                         : "+m" (was_chosen_as_deadlock_victim));
-# else
     was_chosen_as_deadlock_victim.fetch_or(2);
-# endif
   }
 #else /* defined(UNIV_DEBUG) || !defined(DBUG_OFF) */
 
   /** High priority WSREP thread has marked this trx to abort or
-  another transaction chose this as a victim in deadlock resolution. */
+  another transaction chose this as a victim in deadlock resolution.
+
+  Other threads than the one that is executing the transaction may set
+  this while holding lock_sys.wait_mutex. */
   Atomic_relaxed<bool> was_chosen_as_deadlock_victim;
 
   /** Flag the lock owner as a victim in Galera conflict resolution. */
-  void set_wsrep_victim() {
-    was_chosen_as_deadlock_victim= true;
-  }
+  void set_wsrep_victim() { was_chosen_as_deadlock_victim= true; }
 #endif /* defined(UNIV_DEBUG) || !defined(DBUG_OFF) */
 
   /** Next available rec_pool[] entry */
@@ -408,6 +400,9 @@ struct trx_lock_t
 
   /** number of record locks; protected by lock_sys.assert_locked(page_id) */
   ulint n_rec_locks;
+  /** number of lock_rec_set_nth_bit() calls since the start of transaction;
+  protected by lock_sys.is_writer() or trx->mutex_is_owner(). */
+  ulint set_nth_bit_calls;
 };
 
 /** Logical first modification time of a table in a transaction */
@@ -430,9 +425,15 @@ class trx_mod_table_time_t
   /** First modification of a system versioned column
   (NONE= no versioning, BULK= the table was dropped) */
   undo_no_t first_versioned= NONE;
+#ifdef UNIV_DEBUG
+  /** Whether the modified table is a FTS auxiliary table */
+  bool fts_aux_table= false;
+#endif /* UNIV_DEBUG */
 
   /** Buffer to store insert opertion */
   row_merge_bulk_t *bulk_store= nullptr;
+
+  friend struct trx_t;
 public:
   /** Constructor
   @param rows   number of modified rows so far */
@@ -494,6 +495,19 @@ public:
     return false;
   }
 
+#ifdef UNIV_DEBUG
+  void set_aux_table() { fts_aux_table= true; }
+
+  bool is_aux_table() const { return fts_aux_table; }
+#endif /* UNIV_DEBUG */
+
+  /** @return the first undo record that modified the table */
+  undo_no_t get_first() const
+  {
+    ut_ad(valid());
+    return LIMIT & first;
+  }
+
   /** Add the tuple to the transaction bulk buffer for the given index.
   @param entry  tuple to be inserted
   @param index  bulk insert for the index
@@ -506,22 +520,19 @@ public:
 
   /** Do bulk insert operation present in the buffered operation
   @return DB_SUCCESS or error code */
-  dberr_t write_bulk(dict_table_t *table, trx_t *trx)
-  {
-    if (!bulk_store)
-      return DB_SUCCESS;
-    dberr_t err= bulk_store->write_to_table(table, trx);
-    delete bulk_store;
-    bulk_store= nullptr;
-    return err;
-  }
+  dberr_t write_bulk(dict_table_t *table, trx_t *trx);
 
   /** @return whether the buffer storage exist */
-  bool bulk_buffer_exist()
+  bool bulk_buffer_exist() const
   {
-    if (is_bulk_insert() && bulk_store)
-      return true;
-    return false;
+    return bulk_store && is_bulk_insert();
+  }
+
+  /** Free bulk insert operation */
+  void clear_bulk_buffer()
+  {
+    delete bulk_store;
+    bulk_store= nullptr;
   }
 };
 
@@ -610,15 +621,20 @@ struct trx_t : ilist_node<>
 {
 private:
   /**
-    Count of references.
+    Least significant 31 bits is count of references.
 
     We can't release the locks nor commit the transaction until this reference
     is 0. We can change the state to TRX_STATE_COMMITTED_IN_MEMORY to signify
     that it is no longer "active".
-  */
 
+    If the most significant bit is set this transaction should stop inheriting
+    (GAP)locks. Generally set to true during transaction prepare for RC or lower
+    isolation, if requested. Needed for replication replay where
+    we don't want to get blocked on GAP locks taken for protecting
+    concurrent unique insert or replace operation.
+  */
   alignas(CPU_LEVEL1_DCACHE_LINESIZE)
-  Atomic_counter<int32_t> n_ref;
+  Atomic_relaxed<uint32_t> skip_lock_inheritance_and_n_ref;
 
 
 public:
@@ -628,6 +644,10 @@ public:
   Cleared in commit_in_memory() after commit_state(),
   trx_sys_t::deregister_rw(), release_locks(). */
   trx_id_t id;
+  /** The largest encountered transaction identifier for which no
+  transaction was observed to be active. This is a cache to speed up
+  trx_sys_t::find_same_or_older(). */
+  trx_id_t max_inactive_id;
 
 private:
   /** mutex protecting state and some of lock
@@ -760,13 +780,19 @@ public:
 	const char*	op_info;	/*!< English text describing the
 					current operation, or an empty
 					string */
-	uint		isolation_level;/*!< TRX_ISO_REPEATABLE_READ, ... */
-	bool		check_foreigns;	/*!< normally TRUE, but if the user
-					wants to suppress foreign key checks,
-					(in table imports, for example) we
-					set this FALSE */
+  /** TRX_ISO_REPEATABLE_READ, ... */
+  unsigned isolation_level:2;
+  /** when set, REPEATABLE READ will actually be Snapshot Isolation, due to
+  detecting write/write conflicts and disabling "semi-consistent read" */
+  unsigned snapshot_isolation:1;
+  /** normally set; "SET foreign_key_checks=0" can be issued to suppress
+  foreign key checks, in table imports, for example */
+  unsigned check_foreigns:1;
+  /** normally set; "SET unique_checks=0, foreign_key_checks=0"
+  enables bulk insert into an empty table */
+  unsigned check_unique_secondary:1;
   /** whether an insert into an empty table is active */
-  bool bulk_insert;
+  unsigned bulk_insert:1;
 	/*------------------------------*/
 	/* MySQL has a transaction coordinator to coordinate two phase
 	commit between multiple storage engines and the binary log. When
@@ -780,25 +806,12 @@ public:
 	/** whether this is holding the prepare mutex */
 	bool		active_commit_ordered;
 	/*------------------------------*/
-	bool		check_unique_secondary;
-					/*!< normally TRUE, but if the user
-					wants to speed up inserts by
-					suppressing unique key checks
-					for secondary indexes when we decide
-					if we can use the insert buffer for
-					them, we set this FALSE */
 	bool		flush_log_later;/* In 2PC, we hold the
 					prepare_commit mutex across
 					both phases. In that case, we
 					defer flush of the logs to disk
 					until after we release the
 					mutex. */
-	bool		must_flush_log_later;/*!< set in commit()
-					if flush_log_later was
-					set and redo log was written;
-					in that case we will
-					flush the log in
-					trx_commit_complete_for_mysql() */
 	ulint		duplicates;	/*!< TRX_DUP_IGNORE | TRX_DUP_REPLACE */
   /** whether this modifies InnoDB dictionary tables */
   bool dict_operation;
@@ -837,11 +850,13 @@ public:
 					/*!< how many tables the current SQL
 					statement uses, except those
 					in consistent read */
-	dberr_t		error_state;	/*!< 0 if no error, otherwise error
-					number; NOTE That ONLY the thread
-					doing the transaction is allowed to
-					set this field: this is NOT protected
-					by any mutex */
+
+  /** DB_SUCCESS or error code; usually only the thread that is running
+  the transaction is allowed to modify this field. The only exception is
+  when a thread invokes lock_sys_t::cancel() in order to abort a
+  lock_wait(). That is protected by lock_sys.wait_mutex and lock.wait_lock. */
+  dberr_t error_state;
+
 	const dict_index_t*error_info;	/*!< if the error number indicates a
 					duplicate key error, a pointer to
 					the problematic index is stored here */
@@ -990,11 +1005,19 @@ private:
   /** Commit the transaction in a mini-transaction.
   @param mtr  mini-transaction (if there are any persistent modifications) */
   void commit_low(mtr_t *mtr= nullptr);
+  /** Commit an empty transaction.
+  @param mtr   mini-transaction */
+  void commit_empty(mtr_t *mtr);
+  /** Commit an empty transaction.
+  @param mtr   mini-transaction */
+  /** Assign the transaction its history serialisation number and write the
+  UNDO log to the assigned rollback segment.
+  @param mtr   mini-transaction */
+  inline void write_serialisation_history(mtr_t *mtr);
 public:
   /** Commit the transaction. */
   void commit();
 
-
   /** Try to drop a persistent table.
   @param table       persistent table
   @param fk          whether to drop FOREIGN KEY metadata
@@ -1023,26 +1046,40 @@ public:
   void savepoints_discard(trx_named_savept_t *savept);
 
 
-  bool is_referenced() const { return n_ref > 0; }
+  bool is_referenced() const
+  {
+    return (skip_lock_inheritance_and_n_ref & ~(1U << 31)) > 0;
+  }
 
 
   void reference()
   {
-#ifdef UNIV_DEBUG
-    auto old_n_ref=
-#endif
-    n_ref++;
-    ut_ad(old_n_ref >= 0);
+    ut_d(auto old_n_ref =)
+    skip_lock_inheritance_and_n_ref.fetch_add(1);
+    ut_ad(int32_t(old_n_ref << 1) >= 0);
   }
 
-
   void release_reference()
   {
-#ifdef UNIV_DEBUG
-    auto old_n_ref=
-#endif
-    n_ref--;
-    ut_ad(old_n_ref > 0);
+    ut_d(auto old_n_ref =)
+    skip_lock_inheritance_and_n_ref.fetch_sub(1);
+    ut_ad(int32_t(old_n_ref << 1) > 0);
+  }
+
+  bool is_not_inheriting_locks() const
+  {
+    return skip_lock_inheritance_and_n_ref >> 31;
+  }
+
+  void set_skip_lock_inheritance()
+  {
+    ut_d(auto old_n_ref=) skip_lock_inheritance_and_n_ref.fetch_add(1U << 31);
+    ut_ad(!(old_n_ref >> 31));
+  }
+
+  void reset_skip_lock_inheritance()
+  {
+    skip_lock_inheritance_and_n_ref.fetch_and(~1U << 31);
   }
 
   /** @return whether the table has lock on
@@ -1057,6 +1094,7 @@ public:
   {
     ut_ad(state == TRX_STATE_NOT_STARTED);
     ut_ad(!id);
+    ut_ad(!*detailed_error);
     ut_ad(!mutex_is_owner());
     ut_ad(!has_logged());
     ut_ad(!is_referenced());
@@ -1072,6 +1110,9 @@ public:
     ut_ad(UT_LIST_GET_LEN(lock.evicted_tables) == 0);
     ut_ad(!dict_operation);
     ut_ad(!apply_online_log);
+    ut_ad(!is_not_inheriting_locks());
+    ut_ad(check_foreigns);
+    ut_ad(check_unique_secondary);
   }
 
   /** This has to be invoked on SAVEPOINT or at the end of a statement.
@@ -1118,8 +1159,8 @@ public:
   {
     if (UNIV_LIKELY(!bulk_insert))
       return nullptr;
-    ut_ad(!check_unique_secondary);
-    ut_ad(!check_foreigns);
+    ut_ad(table->skip_alter_undo || !check_unique_secondary);
+    ut_ad(table->skip_alter_undo || !check_foreigns);
     auto it= mod_tables.find(table);
     if (it == mod_tables.end() || !it->second.bulk_buffer_exist())
       return nullptr;
@@ -1131,18 +1172,15 @@ public:
   @return DB_SUCCESS or error code */
   dberr_t bulk_insert_apply()
   {
-    if (UNIV_LIKELY(!bulk_insert))
-      return DB_SUCCESS;
-    ut_ad(!check_unique_secondary);
-    ut_ad(!check_foreigns);
-    for (auto& t : mod_tables)
-      if (t.second.is_bulk_insert())
-        if (dberr_t err= t.second.write_bulk(t.first, this))
-          return err;
-    return DB_SUCCESS;
+    return UNIV_UNLIKELY(bulk_insert) ? bulk_insert_apply_low(): DB_SUCCESS;
   }
 
 private:
+  /** Apply the buffered bulk inserts. */
+  dberr_t bulk_insert_apply_low();
+
+  /** Rollback the bulk insert operation for the transaction */
+  void bulk_rollback_low();
   /** Assign a rollback segment for modifying temporary tables.
   @return the assigned rollback segment */
   trx_rseg_t *assign_temp_rseg();
