@@ -470,6 +470,8 @@ func_exit:
 	return(err);
 }
 
+bool dtuple_coll_eq(const dtuple_t &tuple1, const dtuple_t &tuple2);
+
 /** Find out if an accessible version of a clustered index record
 corresponds to a secondary index entry.
 @param rec    record in a latched clustered index page
@@ -591,7 +593,7 @@ nochange_index:
 			a char field, but the collation identifies the old
 			and new value anyway! */
 
-			if (entry && !dtuple_coll_cmp(ientry, entry)) {
+			if (entry && dtuple_coll_eq(*ientry, *entry)) {
 				break;
 			}
 		}
