@@ -256,7 +256,8 @@ trx_purge_add_undo_to_history(const trx_t* trx, trx_undo_t*& undo, mtr_t* mtr)
   if (trx->mysql_log_file_name && *trx->mysql_log_file_name)
     /* Update the latest binlog name and offset if log_bin=ON or this
     is a replica. */
-    trx_rseg_update_binlog_offset(rseg_header, trx, mtr);
+    trx_rseg_update_binlog_offset(rseg_header, trx->mysql_log_file_name,
+                                  trx->mysql_log_offset, mtr);
 
   /* Add the log as the first in the history list */
 
@@ -579,7 +580,7 @@ inline void trx_sys_t::undo_truncate_start(fil_space_t &space)
     }
 }
 
-inline fil_space_t *purge_sys_t::undo_truncate_try(ulint id, ulint size)
+inline fil_space_t *purge_sys_t::undo_truncate_try(uint32_t id, uint32_t size)
 {
   ut_ad(srv_is_undo_tablespace(id));
   fil_space_t *space= fil_space_get(id);
@@ -603,7 +604,7 @@ fil_space_t *purge_sys_t::truncating_tablespace()
   const uint32_t size=
     uint32_t(std::min(ulonglong{std::numeric_limits<uint32_t>::max()},
                       srv_max_undo_log_size >> srv_page_size_shift));
-  for (ulint i= truncate_undo_space.last, j= i;; )
+  for (uint32_t i= truncate_undo_space.last, j= i;; )
   {
     if (fil_space_t *s= undo_truncate_try(srv_undo_space_id_start + i, size))
       return s;
