@@ -59,8 +59,6 @@ public:
   const char         *mem_calc_func_name;
   const char         *mem_calc_file_name;
   ulong              mem_calc_line_no;
-  uint               sql_kinds;
-  uint               *sql_kind;
   ulonglong          *connection_ids;
   uint               conn_kinds;
   uint               *conn_kind;
@@ -81,9 +79,7 @@ public:
   spider_string      *blob_buff;
   SPIDER_POSITION    *pushed_pos;
   SPIDER_POSITION    pushed_pos_buf;
-#ifdef WITH_PARTITION_STORAGE_ENGINE
   SPIDER_PARTITION_HANDLER *partition_handler;
-#endif
   /* Whether this ha_spider is the owner of its wide_handler. */
   bool                wide_handler_owner = FALSE;
   SPIDER_WIDE_HANDLER *wide_handler = NULL;
@@ -109,15 +105,11 @@ public:
   bool               mrr_with_cnt;
   uint               multi_range_cnt;
   uint               multi_range_hit_point;
-#ifdef HA_MRR_USE_DEFAULT_IMPL
   int                multi_range_num;
   bool               have_second_range;
   KEY_MULTI_RANGE    mrr_second_range;
   spider_string      *mrr_key_buff;
   range_id_t         *multi_range_keys;
-#else
-  KEY_MULTI_RANGE    *multi_range_ranges;
-#endif
 
   char               *append_tblnm_alias;
   uint               append_tblnm_alias_length;
@@ -145,15 +137,6 @@ public:
 
   ulonglong          *db_request_id;
   uchar              *db_request_phase;
-  uchar              *m_handler_opened;
-  /* ids for use in HANDLER command */
-  uint               *m_handler_id;
-  /*
-    aliases for use in HANDLER command, in the format of t%5u on
-    m_handler_id. So for example, if m_handler_id is 3, then the
-    corresponding m_handler_cid is t00003
-  */
-  char               **m_handler_cid;
   bool               do_direct_update;
   uint               direct_update_kinds;
   spider_index_rnd_init prev_index_rnd_init;
@@ -221,10 +204,7 @@ public:
   int extra(
     enum ha_extra_function operation
   ) override;
-  int index_init(
-    uint idx,
-    bool sorted
-  ) override;
+  int index_init(uint idx, bool sorted) override;
   int index_end() override;
   int index_read_map(
     uchar *buf,
@@ -263,7 +243,6 @@ public:
   int read_range_next() override;
   void reset_no_where_cond();
   bool check_no_where_cond();
-#ifdef HA_MRR_USE_DEFAULT_IMPL
   ha_rows multi_range_read_info_const(
     uint keyno,
     RANGE_SEQ_IF *seq,
@@ -298,21 +277,7 @@ public:
   int multi_range_read_next_next(
     range_id_t *range_info
   );
-#else
-  int read_multi_range_first(
-    KEY_MULTI_RANGE **found_range_p,
-    KEY_MULTI_RANGE *ranges,
-    uint range_count,
-    bool sorted,
-    HANDLER_BUFFER *buffer
-  );
-  int read_multi_range_next(
-    KEY_MULTI_RANGE **found_range_p
-  );
-#endif
-  int rnd_init(
-    bool scan
-  ) override;
+  int rnd_init(bool scan) override;
   int rnd_end() override;
   int rnd_next(
     uchar *buf
@@ -351,20 +316,7 @@ public:
     key_part_map keypart_map,
     bool use_parallel
   ) override;
-#ifdef HA_MRR_USE_DEFAULT_IMPL
-  int pre_multi_range_read_next(
-    bool use_parallel
-  ) override;
-#else
-  int pre_read_multi_range_first(
-    KEY_MULTI_RANGE **found_range_p,
-    KEY_MULTI_RANGE *ranges,
-    uint range_count,
-    bool sorted,
-    HANDLER_BUFFER *buffer,
-    bool use_parallel
-  );
-#endif
+  int pre_multi_range_read_next(bool use_parallel) override;
   int pre_read_range_first(
     const key_range *start_key,
     const key_range *end_key,
@@ -391,14 +343,8 @@ public:
   const char *table_type() const override;
   ulonglong table_flags() const override;
   ulong table_flags_for_partition();
-  const char *index_type(
-    uint key_number
-  ) override;
-  ulong index_flags(
-    uint idx,
-    uint part,
-    bool all_parts
-  ) const override;
+  const char *index_type(uint key_number) override;
+  ulong index_flags(uint idx, uint part, bool all_parts) const override;
   uint max_supported_record_length() const override;
   uint max_supported_keys() const override;
   uint max_supported_key_parts() const override;
@@ -415,26 +361,17 @@ public:
     ulonglong *first_value,
     ulonglong *nb_reserved_values
   ) override;
-  int reset_auto_increment(
-    ulonglong value
-  ) override;
+  int reset_auto_increment(ulonglong value) override;
   void release_auto_increment() override;
-  void start_bulk_insert(
-    ha_rows rows,
-    uint flags
-  ) override;
+  void start_bulk_insert(ha_rows rows, uint flags) override;
   int end_bulk_insert() override;
-  int write_row(
-    const uchar *buf
-  ) override;
+  int write_row(const uchar *buf) override;
   void direct_update_init(
     THD *thd,
     bool hs_request
   );
   bool start_bulk_update() override;
-  int exec_bulk_update(
-    ha_rows *dup_key_found
-  ) override;
+  int exec_bulk_update(ha_rows *dup_key_found) override;
   int end_bulk_update() override;
   int bulk_update_row(
     const uchar *old_data,
@@ -450,84 +387,18 @@ public:
     longlong select_limit,
     longlong offset_limit
   );
-#ifdef HANDLER_HAS_DIRECT_UPDATE_ROWS_WITH_HS
-  inline int direct_update_rows_init(
-    List<Item> *update_fields
-  ) {
-    return direct_update_rows_init(update_fields, 2, NULL, 0, FALSE, NULL);
-  }
-  int direct_update_rows_init(
-    List<Item> *update_fields,
-    uint mode,
-    KEY_MULTI_RANGE *ranges,
-    uint range_count,
-    bool sorted,
-    uchar *new_data
-  );
-#else
-  int direct_update_rows_init(
-    List<Item> *update_fields
-  ) override;
-#endif
-#ifdef HANDLER_HAS_DIRECT_UPDATE_ROWS_WITH_HS
-  inline int direct_update_rows(ha_rows *update_rows, ha_rows *found_rows)
-  {
-    return direct_update_rows(NULL, 0, FALSE, NULL, update_rows, found_rows);
-  }
-  int direct_update_rows(
-    KEY_MULTI_RANGE *ranges,
-    uint range_count,
-    bool sorted,
-    uchar *new_data,
-    ha_rows *update_rows,
-    ha_rows *found_row
-  );
-#else
-  int direct_update_rows(
-    ha_rows *update_rows,
-    ha_rows *found_row
-  ) override;
-#endif
+  int direct_update_rows_init(List<Item> *update_fields) override;
+  int direct_update_rows(ha_rows *update_rows, ha_rows *found_row) override;
   bool start_bulk_delete() override;
   int end_bulk_delete() override;
-  int delete_row(
-    const uchar *buf
-  ) override;
+  int delete_row(const uchar *buf) override;
   bool check_direct_delete_sql_part(
     st_select_lex *select_lex,
     longlong select_limit,
     longlong offset_limit
   );
-#ifdef HANDLER_HAS_DIRECT_UPDATE_ROWS_WITH_HS
-  inline int direct_delete_rows_init()
-  {
-    return direct_delete_rows_init(2, NULL, 0, FALSE);
-  }
-  int direct_delete_rows_init(
-    uint mode,
-    KEY_MULTI_RANGE *ranges,
-    uint range_count,
-    bool sorted
-  );
-#else
   int direct_delete_rows_init() override;
-#endif
-#ifdef HANDLER_HAS_DIRECT_UPDATE_ROWS_WITH_HS
-  inline int direct_delete_rows(ha_rows *delete_rows)
-  {
-    return direct_delete_rows(NULL, 0, FALSE, delete_rows);
-  }
-  int direct_delete_rows(
-    KEY_MULTI_RANGE *ranges,
-    uint range_count,
-    bool sorted,
-    ha_rows *delete_rows
-  );
-#else
-  int direct_delete_rows(
-    ha_rows *delete_rows
-  ) override;
-#endif
+  int direct_delete_rows(ha_rows *delete_rows) override;
   int delete_all_rows() override;
   int truncate() override;
   double scan_time() override;
@@ -603,10 +474,7 @@ public:
     const COND* cond
   ) override;
   void cond_pop() override;
-  int info_push(
-    uint info_type,
-    void *info
-  ) override;
+  int info_push(uint info_type, void *info) override;
   void return_record_by_parent() override;
   TABLE *get_table();
   void set_ft_discard_bitmap();
@@ -614,9 +482,7 @@ public:
   void set_clone_searched_bitmap();
   void set_searched_bitmap_from_item_list();
   void set_select_column_mode();
-#ifdef WITH_PARTITION_STORAGE_ENGINE
   void check_select_column(bool rnd);
-#endif
   bool check_and_start_bulk_update(
     spider_bulk_upd_start bulk_upd_start
   );
@@ -629,25 +495,7 @@ public:
   bool is_sole_projection_field(
     uint16 field_index
   );
-  int check_ha_range_eof();
   int drop_tmp_tables();
-  bool handler_opened(
-    int link_idx,
-    uint tgt_conn_kind
-  );
-  void set_handler_opened(
-    int link_idx
-  );
-  void clear_handler_opened(
-    int link_idx,
-    uint tgt_conn_kind
-  );
-  int close_opened_handler(
-    int link_idx,
-    bool release_conn
-  );
-  int index_handler_init();
-  int rnd_handler_init();
   void set_error_mode();
   void backup_error_status();
   int check_error_mode(
@@ -676,17 +524,6 @@ public:
     bool eq_range,
     bool sorted
   );
-#ifdef HA_MRR_USE_DEFAULT_IMPL
-#else
-  int read_multi_range_first_internal(
-    uchar *buf,
-    KEY_MULTI_RANGE **found_range_p,
-    KEY_MULTI_RANGE *ranges,
-    uint range_count,
-    bool sorted,
-    HANDLER_BUFFER *buffer
-  );
-#endif
   int ft_read_internal(uchar *buf);
   int rnd_next_internal(uchar *buf);
   void check_pre_call(
@@ -902,9 +739,6 @@ public:
     const char *alias,
     uint alias_length
   );
-  bool support_use_handler_sql(
-    int use_handler
-  );
   int init_union_table_name_pos_sql();
   int set_union_table_name_pos_sql();
   int append_lock_tables_list();
