@@ -14,8 +14,8 @@
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1335 USA */
 
-#define SPIDER_DETAIL_VERSION "3.3.15"
-#define SPIDER_HEX_VERSION 0x0303
+#define SPIDER_DETAIL_VERSION PACKAGE_VERSION
+#define SPIDER_HEX_VERSION (MYSQL_VERSION_MAJOR << 8 | MYSQL_VERSION_MINOR)
 
 #define spider_my_free(A,B) my_free(A)
 #ifdef pthread_mutex_t
@@ -66,7 +66,6 @@
 #define pthread_cond_destroy mysql_cond_destroy
 #define my_sprintf(A,B) sprintf B
 
-
 #define spider_stmt_da_message(A) thd_get_error_message(A)
 #define spider_stmt_da_sql_errno(A) thd_get_error_number(A)
 #define spider_user_defined_key_parts(A) (A)->user_defined_key_parts
@@ -88,7 +87,6 @@
 
 #define SPIDER_FIELD_FIELDPTR_REQUIRES_THDPTR
 #define SPIDER_ENGINE_CONDITION_PUSHDOWN_IS_ALWAYS_ON
-#define SPIDER_XID_USES_xid_cache_iterate
 
 #define SPIDER_Item_func_conv_charset_conv_charset collation.collation
 
@@ -98,7 +96,6 @@
 #define SPIDER_new_THD(A) (new THD(A))
 #define SPIDER_order_direction_is_asc(A) (A->direction == ORDER::ORDER_ASC)
 
-#define SPIDER_HAS_MY_CHARLEN
 #define SPIDER_open_temporary_table
 
 #define SPIDER_generate_partition_syntax(A,B,C,D,E,F,G,H) generate_partition_syntax(A,B,C,E,F,G)
@@ -163,7 +160,7 @@ typedef start_new_trans *SPIDER_Open_tables_backup;
 
 #define SPIDER_TMP_SHARE_CHAR_PTR_COUNT     23
 #define SPIDER_TMP_SHARE_UINT_COUNT         SPIDER_TMP_SHARE_CHAR_PTR_COUNT
-#define SPIDER_TMP_SHARE_LONG_COUNT         20
+#define SPIDER_TMP_SHARE_LONG_COUNT         19
 #define SPIDER_TMP_SHARE_LONGLONG_COUNT      3
 
 #define SPIDER_MEM_CALC_LIST_NUM           SPD_MID_LAST
@@ -465,7 +462,6 @@ typedef struct st_spider_share SPIDER_SHARE;
 typedef struct st_spider_table_mon_list SPIDER_TABLE_MON_LIST;
 typedef struct st_spider_ip_port_conn SPIDER_IP_PORT_CONN;
 
-#ifndef WITHOUT_SPIDER_BG_SEARCH
 typedef struct st_spider_thread
 {
   uint                  thread_idx;
@@ -481,7 +477,6 @@ typedef struct st_spider_thread
   volatile SPIDER_SHARE *queue_first;
   volatile SPIDER_SHARE *queue_last;
 } SPIDER_THREAD;
-#endif
 
 typedef struct st_spider_link_for_hash
 {
@@ -694,11 +689,8 @@ typedef struct st_spider_conn
   uint               tgt_driver_length;
   uint               dbton_id;
 
-#ifndef WITHOUT_SPIDER_BG_SEARCH
   volatile
-#endif
     void             *quick_target;
-#ifndef WITHOUT_SPIDER_BG_SEARCH
   volatile bool      bg_init;
   volatile bool      bg_break;
   volatile bool      bg_kill;
@@ -729,10 +721,7 @@ typedef struct st_spider_conn
   const char         *bg_job_stack_file_name;
   ulong              bg_job_stack_line_no;
   uint               bg_job_stack_cur_pos;
-#endif
-#ifndef WITHOUT_SPIDER_BG_SEARCH
   volatile
-#endif
     int              *need_mon;
   int                *conn_need_mon;
 
@@ -811,7 +800,6 @@ typedef struct st_spider_lgtm_tblhnd_share
   ulonglong          auto_increment_value;
 } SPIDER_LGTM_TBLHND_SHARE;
 
-#ifdef WITH_PARTITION_STORAGE_ENGINE
 typedef struct st_spider_patition_handler
 {
   bool               clone_bitmap_init;
@@ -821,7 +809,6 @@ typedef struct st_spider_patition_handler
   ha_spider          *owner;
   ha_spider          **handlers;
 } SPIDER_PARTITION_HANDLER;
-#endif
 
 typedef struct st_spider_wide_share
 {
@@ -874,9 +861,7 @@ typedef struct st_spider_wide_handler
   uchar              *rnd_write_bitmap;
   SPIDER_CONDITION   *condition;
   void               *owner;
-#ifdef WITH_PARTITION_STORAGE_ENGINE
   SPIDER_PARTITION_HANDLER *partition_handler;
-#endif
   List<Item>         *direct_update_fields;
   List<Item>         *direct_update_values;
   TABLE_SHARE        *top_share;
@@ -927,7 +912,6 @@ typedef struct st_spider_transaction
   bool               updated_in_this_trx;
 
   THD                *thd;
-  my_hash_value_type thd_hash_value;
   XID                xid;
   HASH               trx_conn_hash;
   uint               trx_conn_hash_id;
@@ -1019,9 +1003,7 @@ typedef struct st_spider_share
   TABLE_SHARE        *table_share;
   SPIDER_LGTM_TBLHND_SHARE *lgtm_tblhnd_share;
   my_hash_value_type table_name_hash_value;
-#ifdef WITH_PARTITION_STORAGE_ENGINE
   my_hash_value_type table_path_hash_value;
-#endif
 
   volatile bool      init;
   volatile bool      init_error;
@@ -1030,13 +1012,10 @@ typedef struct st_spider_share
   uchar              *table_mon_mutex_bitmap;
   volatile bool      sts_init;
   volatile time_t    sts_get_time;
-#ifndef WITHOUT_SPIDER_BG_SEARCH
   volatile time_t    bg_sts_try_time;
   volatile double    bg_sts_interval;
   volatile int       bg_sts_mode;
-#ifdef WITH_PARTITION_STORAGE_ENGINE
   volatile int       bg_sts_sync;
-#endif
   volatile bool      bg_sts_init;
   volatile bool      bg_sts_kill;
   volatile bool      bg_sts_thd_wait;
@@ -1045,15 +1024,11 @@ typedef struct st_spider_share
   pthread_cond_t     bg_sts_cond;
   pthread_cond_t     bg_sts_sync_cond;
   volatile bool      crd_init;
-#endif
   volatile time_t    crd_get_time;
-#ifndef WITHOUT_SPIDER_BG_SEARCH
   volatile time_t    bg_crd_try_time;
   volatile double    bg_crd_interval;
   volatile int       bg_crd_mode;
-#ifdef WITH_PARTITION_STORAGE_ENGINE
   volatile int       bg_crd_sync;
-#endif
   volatile bool      bg_crd_init;
   volatile bool      bg_crd_kill;
   volatile bool      bg_crd_thd_wait;
@@ -1061,8 +1036,6 @@ typedef struct st_spider_share
   pthread_t          bg_crd_thread;
   pthread_cond_t     bg_crd_cond;
   pthread_cond_t     bg_crd_sync_cond;
-#endif
-#ifndef WITHOUT_SPIDER_BG_SEARCH
   volatile bool      bg_mon_init;
   volatile bool      bg_mon_kill;
   THD                **bg_mon_thds;
@@ -1070,8 +1043,6 @@ typedef struct st_spider_share
   pthread_mutex_t    *bg_mon_mutexes;
   pthread_cond_t     *bg_mon_conds;
   pthread_cond_t     *bg_mon_sleep_conds;
-#endif
-#ifndef WITHOUT_SPIDER_BG_SEARCH
   /* static bg thread for sts and crd */
   TABLE                 table;
   ha_spider             *sts_spider;
@@ -1088,7 +1059,6 @@ typedef struct st_spider_share
   volatile SPIDER_SHARE *sts_next;
   volatile SPIDER_SHARE *crd_prev;
   volatile SPIDER_SHARE *crd_next;
-#endif
 
   MEM_ROOT           mem_root;
 
@@ -1110,24 +1080,16 @@ typedef struct st_spider_share
   longlong           additional_table_flags;
   bool               have_recovery_link;
 
-#ifndef WITHOUT_SPIDER_BG_SEARCH
   int                sts_bg_mode;
-#endif
   double             sts_interval;
   int                sts_mode;
-#ifdef WITH_PARTITION_STORAGE_ENGINE
   int                sts_sync;
-#endif
   int                store_last_sts;
   int                load_sts_at_startup;
-#ifndef WITHOUT_SPIDER_BG_SEARCH
   int                crd_bg_mode;
-#endif
   double             crd_interval;
   int                crd_mode;
-#ifdef WITH_PARTITION_STORAGE_ENGINE
   int                crd_sync;
-#endif
   int                store_last_crd;
   int                load_crd_at_startup;
   int                crd_type;
@@ -1161,11 +1123,9 @@ typedef struct st_spider_share
   int                low_mem_read;
   int                table_count_mode;
   int                select_column_mode;
-#ifndef WITHOUT_SPIDER_BG_SEARCH
   int                bgs_mode;
   longlong           bgs_first_read;
   longlong           bgs_second_read;
-#endif
   longlong           first_read;
   longlong           second_read;
   int                auto_increment_mode;
@@ -1218,19 +1178,14 @@ typedef struct st_spider_share
   long               *tgt_ports;
   long               *tgt_ssl_vscs;
   long               *link_statuses;
-#ifndef WITHOUT_SPIDER_BG_SEARCH
   long               *monitoring_bg_flag;
   long               *monitoring_bg_kind;
-#endif
   long               *monitoring_binlog_pos_at_failing;
   long               *monitoring_flag;
   long               *monitoring_kind;
-#ifndef WITHOUT_SPIDER_BG_SEARCH
   longlong           *monitoring_bg_interval;
-#endif
   longlong           *monitoring_limit;
   longlong           *monitoring_sid;
-  long               *use_handlers;
   long               *connect_timeouts;
   long               *net_read_timeouts;
   long               *net_write_timeouts;
@@ -1311,19 +1266,14 @@ typedef struct st_spider_share
   uint               tgt_ports_length;
   uint               tgt_ssl_vscs_length;
   uint               link_statuses_length;
-#ifndef WITHOUT_SPIDER_BG_SEARCH
   uint               monitoring_bg_flag_length;
   uint               monitoring_bg_kind_length;
-#endif
   uint               monitoring_binlog_pos_at_failing_length;
   uint               monitoring_flag_length;
   uint               monitoring_kind_length;
-#ifndef WITHOUT_SPIDER_BG_SEARCH
   uint               monitoring_bg_interval_length;
-#endif
   uint               monitoring_limit_length;
   uint               monitoring_sid_length;
-  uint               use_handlers_length;
   uint               connect_timeouts_length;
   uint               net_read_timeouts_length;
   uint               net_write_timeouts_length;
@@ -1518,9 +1468,7 @@ typedef struct st_spider_copy_table_conn
   spider_db_copy_table       *copy_table;
   ha_spider                  *spider;
   int                        need_mon;
-#ifndef WITHOUT_SPIDER_BG_SEARCH
   int                        bg_error_num;
-#endif
   st_spider_copy_table_conn  *next;
 } SPIDER_COPY_TABLE_CONN;
 
@@ -1545,9 +1493,7 @@ typedef struct st_spider_copy_tables
   longlong                   bulk_insert_rows;
   int                        use_table_charset;
   int                        use_transaction;
-#ifndef WITHOUT_SPIDER_BG_SEARCH
   int                        bg_mode;
-#endif
 
   char                       *database;
 
@@ -1612,10 +1558,8 @@ typedef struct st_spider_item_hld
 {
   uint               tgt_num;
   Item               *item;
-#ifdef SPIDER_ITEM_STRING_WITHOUT_SET_STR_WITH_COPY_AND_THDPTR
   bool               init_mem_root;
   MEM_ROOT           mem_root;
-#endif
   st_spider_item_hld *next;
 } SPIDER_ITEM_HLD;
 
